@@ -1,6 +1,8 @@
 #!/usr/bin/python3.7
 """Instrument speedtest.net speedtests from Prometheus."""
 
+from typing import Optional
+from typing import Sequence
 from http import server
 from urllib.parse import urlparse
 import os
@@ -17,25 +19,35 @@ from prometheus_speedtest import version
 flags.DEFINE_string('address', '0.0.0.0', 'address to listen on')
 flags.DEFINE_integer('port', 9516, 'port to listen on')
 flags.DEFINE_list(
-    'servers', None,
+    'servers', [],
     'speedtest server(s) to use - leave empty for auto-selection')
+flags.DEFINE_list(
+    'excludes', [],
+    'speedtest server(s) to exclude - leave empty for no exclusion')
 flags.DEFINE_boolean('version', False, 'show version')
 FLAGS = flags.FLAGS
 
 
 class PrometheusSpeedtest():
     """Enapsulates behavior performing and reporting results of speedtests."""
-    def __init__(self, source_address=None, timeout=10, servers=None):
+    def __init__(self,
+                 source_address: Optional[str] = None,
+                 timeout: int = 10,
+                 servers: Optional[Sequence[str]] = None,
+                 excludes: Optional[Sequence[str]] = None):
         """Instantiates a PrometheusSpeedtest object.
 
         Args:
             source_address: str - optional network address to bind to.
                 e.g. 192.168.1.1.
             timeout: int - optional timeout for speedtest in seconds.
+            servers: list of speedtest server ids to restrict testing to.
+            servers: list of speedtest server ids to exclude testing from.
         """
         self._source_address = source_address
         self._timeout = timeout
         self._servers = servers
+        self._excludes = excludes
 
     def test(self):
         """Performs speedtest, returns results.
@@ -46,8 +58,9 @@ class PrometheusSpeedtest():
         logging.info('Performing Speedtest...')
         client = speedtest.Speedtest(source_address=self._source_address,
                                      timeout=self._timeout)
-        logging.debug('Eligible servers: %s',
-                      client.get_servers(servers=self._servers))
+        logging.debug(
+            'Eligible servers: %s',
+            client.get_servers(servers=self._servers, exclude=self._excludes))
         logging.debug('Picked server: %s', client.get_best_server())
         client.download()
         client.upload()
@@ -57,7 +70,10 @@ class PrometheusSpeedtest():
 
 class SpeedtestCollector():
     """Performs Speedtests when requested from Prometheus."""
-    def __init__(self, tester=None, servers=None):
+    def __init__(self,
+                 tester: Optional[PrometheusSpeedtest] = None,
+                 servers: Optional[Sequence[str]] = None,
+                 excludes: Optional[Sequence[str]] = None):
         """Instantiates a SpeedtestCollector object.
 
         Args:
@@ -65,7 +81,7 @@ class SpeedtestCollector():
             servers: servers-id to use when tester is auto-created
         """
         self._tester = tester if tester else PrometheusSpeedtest(
-            servers=servers)
+            servers=servers, excludes=excludes)
 
     def collect(self):
         """Performs a Speedtests and yields metrics.
@@ -131,7 +147,12 @@ def main(argv):
         return
 
     registry = core.CollectorRegistry(auto_describe=False)
-    registry.register(SpeedtestCollector(servers=FLAGS.servers))
+    if FLAGS.servers and set(FLAGS.excludes).issuperset(FLAGS.servers):
+        logging.fatal(
+            '--excludes is a superset of --includes, no viable test server is '
+            'possible. Ensure --excludes does not contain all --servers.')
+    registry.register(
+        SpeedtestCollector(servers=FLAGS.servers, excludes=FLAGS.excludes))
     metrics_handler = SpeedtestMetricsHandler.factory(registry)
 
     http = server.ThreadingHTTPServer((FLAGS.address, FLAGS.port),
